package com.ocpsoft.pretty.event;

/*
 * PrettyFaces is an OpenSource JSF library to create bookmarkable URLs.
 * 
 * Copyright (C) 2008 - Lincoln Baxter, III <lincoln@ocpsoft.com>
 * 
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see the file COPYING.LESSER or visit the GNU
 * website at <http://www.gnu.org/licenses/>.
 */
import java.io.IOException;

import javax.faces.application.NavigationHandler;
import javax.faces.application.ViewHandler;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.servlet.http.HttpServletResponse;

import com.ocpsoft.pretty.PrettyContext;
import com.ocpsoft.pretty.PrettyException;
import com.ocpsoft.pretty.beans.ActionExecutor;
import com.ocpsoft.pretty.config.PrettyUrlMapping;
import com.ocpsoft.pretty.util.FacesMessagesUtils;

public class PrettyPhaseListener implements PhaseListener
{
    private static final long serialVersionUID = 2345410822999587673L;
    private final FacesMessagesUtils messagesUtils = new FacesMessagesUtils();
    private final ActionExecutor executor = new ActionExecutor();

    public PhaseId getPhaseId()
    {
        return PhaseId.ANY_PHASE;
    }

    public void beforePhase(final PhaseEvent event)
    {
        FacesContext context = event.getFacesContext();
        PrettyContext prettyContext = PrettyContext.getCurrentInstance();
        if (PhaseId.RESTORE_VIEW.equals(event.getPhaseId()))
        {
            messagesUtils.restoreMessages(context);
        }

        PrettyUrlMapping mapping = prettyContext.getCurrentMapping();
        if (mapping != null)
        {
            String result = executor.executeActions(context, event.getPhaseId(), mapping);
            if (result != null)
            {
                redirect(result);
                return;
            }
        }
    }

    private void redirect(final String toActionId)
    {
        FacesContext context = FacesContext.getCurrentInstance();
        try
        {
            // This sets the ViewRoot
            NavigationHandler navigationHandler = context.getApplication().getNavigationHandler();
            try
            {
                navigationHandler.handleNavigation(context, null, toActionId);
            }
            catch (PrettyException e)
            {
                throw e;
            }
            catch (Exception e)
            {
                HttpServletResponse response = (HttpServletResponse) context.getExternalContext().getResponse();
                if ((context.getViewRoot() != null) && !response.isCommitted())
                {
                    PrettyContext.getCurrentInstance().setInNavigation(true);
                    ViewHandler viewHandler = context.getApplication().getViewHandler();
                    String viewId = viewHandler.getActionURL(context, context.getViewRoot().getViewId());
                    context.getExternalContext().redirect(viewId);
                }
            }
        }
        catch (IOException e)
        {
            throw new PrettyException("PrettyFaces: Error while navigating to action result string<" + toActionId
                    + ">, " + e.getMessage(), e);
        }
    }

    public void afterPhase(final PhaseEvent event)
    {}
}
